<?php

namespace Drupal\social_auth_mailru;

use Drupal\social_auth\AuthManager\OAuth2Manager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Contains all the logic for Mailru login integration.
 */
class MailruAuthManager extends OAuth2Manager {

  /**
   * The Mailru client object.
   *
   * @var \Qzmenko\OAuth2\Client\Provider\Mailru
   */
  protected $client;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Used for logging errors.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   Used for accessing configuration object factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Used to get the authorization code from the callback request.
   */
  public function __construct(
    ConfigFactory $configFactory,
    LoggerChannelFactoryInterface $logger_factory,
    RequestStack $request_stack
  ) {
    parent::__construct(
      $configFactory->get('social_auth_mailru.settings'),
      $logger_factory,
      $request_stack->getCurrentRequest()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate() {
    try {
      $this->setAccessToken(
        $this->client->getAccessToken('authorization_code', [
          'code' => $this->request->query->get('code')
        ])
      );
    }
    catch (IdentityProviderException $e) {
      $this->loggerFactory->get('social_auth_mailru')
        ->error('There was an error during authentication. Exception: ' . $e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getUserInfo() {
    $access_token = $this->getAccessToken();
    if (!$this->user && $access_token != NULL) {
      $this->user = $this->client->getResourceOwner($access_token);
    }
    else {
      $this->loggerFactory->get('social_auth_mailru')
        ->error('There was an error fetching the access token for user.');
    }

    return $this->user;
  }

  /**
   * {@inheritdoc}
   */
  public function getState() {
    return $this->client->getState();
  }

  /**
   * {@inheritdoc}
   */
  public function requestEndPoint($method, $path, $domain = NULL, array $options = []) {
    // TODO: Implement requestEndPoint() method.
  }

  /**
   * {@inheritdoc}
   */
  public function getAuthorizationUrl() {
    // Returns the URL where user will be redirected.
    return $this->client->getAuthorizationUrl();
  }

}
